@@ -32,6 +32,8 @@ REFERENCE = 0
 X_PAGINATION_PAGE_COUNT = "X-Pagination-Page-Count"
 SEGMENT = "targetingSegment"
 KAMELEOON_TRACK_EXPERIMENT_THREAD = "KameleoonTrackExperimentThread"
+KAMELEOON_TRACK_DATA_THREAD = "KameleoonTrackDataThread"
+KAMELEOON_ACTIVATE_FEATURE_THREAD = "KameleoonActivateFeatureThread"
 STATUS_ACTIVE = "ACTIVE"
 FEATURE_STATUS_DEACTIVATED = "DEACTIVATED"
 
@@ -58,6 +60,7 @@ class KameleoonClient:
         kameleoon_client = KameleoonClient(SITE_CODE, blocking=True, logger=MyLogger)
     """
     initialize = False
+    timer: Optional[threading.Timer] = None
 
     # pylint: disable=R0913
     def __init__(
@@ -100,7 +103,11 @@ class KameleoonClient:
 
         self.data: Dict[str, List[Type[Data]]] = {}
 
-        self._fetch_configuration()
+        self._init_fetch_configuration()
+
+    def __del__(self):
+        if self.timer is not None:
+            self.timer.cancel()
 
     def make_sync_request(self, url: str, headers: Dict[str, str],
                           filters: Optional[List[Dict[str, Union[str, List[Union[str, int]]]]]] = None,
@@ -371,20 +378,19 @@ class KameleoonClient:
                 for k, value in experiment['deviations'].items():
                     threshold -= value
                     if threshold < 0:
-                        thread = threading.Thread(target=self.track_experiment, args=[visitor_code, experiment_id, k])
-                        thread.daemon = True
-                        thread.setName(KAMELEOON_TRACK_EXPERIMENT_THREAD)
-                        thread.start()
+                        self._run_in_threads_if_needed(multi_threading=self.multi_threading,
+                                                       func=self.track_experiment,
+                                                       args=[visitor_code, experiment_id, k],
+                                                       thread_name=KAMELEOON_TRACK_EXPERIMENT_THREAD)
                         try:
                             variation = int(k)
                         except ValueError:
                             variation = 0
                         return variation
-                thread = threading.Thread(target=self.track_experiment,
-                                          args=[visitor_code, experiment_id, REFERENCE, True])
-                thread.daemon = True
-                thread.setName(KAMELEOON_TRACK_EXPERIMENT_THREAD)
-                thread.start()
+                self._run_in_threads_if_needed(multi_threading=self.multi_threading,
+                                               func=self.track_experiment,
+                                               args=[visitor_code, experiment_id, REFERENCE, True],
+                                               thread_name=KAMELEOON_TRACK_EXPERIMENT_THREAD)
                 raise NotActivated(message=visitor_code)
             raise NotTargeted(message=visitor_code)
 
@@ -469,7 +475,7 @@ class KameleoonClient:
         self.add_data(visitor_code, Conversion(goal_id, revenue))
         self.flush(visitor_code)
 
-    def flush(self, visitor_code: Optional[str] = None) -> threading.Thread:
+    def flush(self, visitor_code: Optional[str] = None):
         """
         Data associated with the current user via add_data() method is not immediately sent to the server.
         It is stored and accumulated until it is sent automatically by the trigger_experiment()
@@ -483,7 +489,6 @@ class KameleoonClient:
 
         :param visitor_code: Unique identifier of the user. This field is mandatory.
         :type visitor_code: Optional[str]
-        :return: threading.Thread
 
         Examples:
 
@@ -503,11 +508,10 @@ class KameleoonClient:
         """
         if visitor_code is not None:
             check_visitor_code(visitor_code)
-        thread = threading.Thread(target=self.track_data, args=[visitor_code, ])
-        thread.daemon = True
-        thread.setName("KameleoonTrackDataThread")
-        thread.start()
-        return thread
+        self._run_in_threads_if_needed(multi_threading=self.multi_threading,
+                                       func=self.track_data,
+                                       args=[visitor_code, ],
+                                       thread_name=KAMELEOON_TRACK_DATA_THREAD)
 
     def obtain_variation_associated_data(self, variation_id: int) -> Dict[str, str]:
         """ Obtain variation associated data.
@@ -647,20 +651,20 @@ class KameleoonClient:
                     SEGMENT].check_tree(visitor_data):
                 if self._feature_flag_scheduled(feature_flag, time.time()):
                     threshold = obtain_hash_double(visitor_code, {}, feature_flag['id'])
+                    track_info = []
                     if threshold >= 1 - feature_flag['expositionRate']:
                         variations_id = None
                         if feature_flag["variations"]:
                             variations_id = feature_flag["variations"][0]
-                        thread = threading.Thread(target=self.track_experiment, args=[
-                            visitor_code, feature_flag['id'], variations_id])
+                        track_info = [visitor_code, feature_flag['id'], variations_id]
                         activate = True
                     else:
-                        thread = threading.Thread(target=self.track_experiment, args=[
-                            visitor_code, feature_flag['id'], REFERENCE, True])
+                        track_info = [visitor_code, feature_flag['id'], REFERENCE, True]
                         activate = False
-                    thread.daemon = True
-                    thread.setName("KameleoonActivateFeatureThread")
-                    thread.start()
+                    self._run_in_threads_if_needed(multi_threading=self.multi_threading,
+                                                   func=self.track_experiment,
+                                                   args=track_info,
+                                                   thread_name=KAMELEOON_ACTIVATE_FEATURE_THREAD)
                     return activate
                 else:
                     return False
@@ -768,14 +772,11 @@ class KameleoonClient:
             headers=self._get_header_client(),
             timeout=timeout)
 
-    def _fetch_configuration(self) -> None:
+    def _init_fetch_configuration(self) -> None:
         """
         :return:
         """
-        thread = threading.Thread(target=self._fetch_configuration_job)
-        thread.daemon = True
-        thread.setName("KameleoonClientThread")
-        thread.start()
+        self._fetch_graphql()
 
     def _fetch(self) -> None:
         """
@@ -806,20 +807,18 @@ class KameleoonClient:
         self.logger.debug('Scheduled job to fetch configuration graphQL is starting.')
         self.obtain_experiments_graphql(self.site_code)
         self.obtain_feature_flags_graphql(self.site_code, self.environment)
+        if self.timer is not None:
+            self.timer.cancel()
 
-    def _fetch_configuration_job(self) -> None:
+        self._add_fetch_configuration_after_delay()
+
+    def _add_fetch_configuration_after_delay(self) -> None:
         """
         :return: None
         """
-        try:
-            actions_configuration_refresh_interval = int(self.config['actions_configuration_refresh_interval']) * 60
-        except KeyError:
-            actions_configuration_refresh_interval = DEFAULT_CONFIGURATION_UPDATE_INTERVAL
-
-        while True:
-            self._fetch_graphql()
-            # self._fetch()
-            time.sleep(actions_configuration_refresh_interval)
+        self.timer = threading.Timer(self.actions_configuration_refresh_interval, self._fetch_graphql)
+        self.timer.setDaemon(True)
+        self.timer.start()
 
     def _get_header_client(self) -> Dict[str, str]:
         return {
@@ -1220,3 +1219,39 @@ class KameleoonClient:
             self.environment = self.config['environment']
         except KeyError:
             self.environment = 'production'
+        try:
+            self.multi_threading = self.config['multi_threading']
+        except KeyError:
+            self.multi_threading = False
+        try:
+            actions_configuration_refresh_interval = int(self.config['actions_configuration_refresh_interval'])
+            self.actions_configuration_refresh_interval = actions_configuration_refresh_interval * 60
+        except KeyError:
+            self.actions_configuration_refresh_interval = DEFAULT_CONFIGURATION_UPDATE_INTERVAL
+
+    def _run_in_threads_if_needed(self,
+                                  multi_threading: bool,
+                                  func,
+                                  args: List[Any],
+                                  thread_name: str):
+        """
+        it's wrapper function which run the `func`
+        in another thread if multi_threading option is True
+        else it calls the `func` in the same thread
+        :param multi_threading: Flag to determine if run in multi-threading
+        :type multi_threading: Boolean
+        :param func: Function need to be called
+        :type func: Function
+        :param args: List of arguments for `func`
+        :type args: List[Any]
+        :param thread_name: Name of thread if `func` runs
+        in another thread
+        :type args: str
+        """
+        if multi_threading:
+            thread = threading.Thread(target=func, args=args)
+            thread.daemon = True
+            thread.setName(thread_name)
+            thread.start()
+        else:
+            func(*args)
